@@ -89,6 +89,16 @@ get_handler_priority(gint *priority, PyObject *kwargs)
     return 0;
 }
 
+static PyObject *
+pyglib_threads_init(PyObject *unused, PyObject *args, PyObject *kwargs)
+{
+    if (!pyglib_enable_threads())
+        return NULL;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static PyObject *
 pyglib_idle_add(PyObject *self, PyObject *args, PyObject *kwargs)
 {
@@ -585,7 +595,31 @@ pyglib_set_prgname(PyObject *self, PyObject *arg)
     return Py_None;
 }
 
+static PyObject *
+pyglib_find_program_in_path(PyObject *unused, PyObject *args, PyObject *kwargs)
+{
+    static char *kwlist[] = { "program", NULL };
+    char *program, *ret;
+    PyObject *retval;
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs,
+				     "s:glib.find_program_in_path", kwlist,
+                                     &program))
+        return NULL;
+
+    ret = g_find_program_in_path(program);
+    retval = _PyUnicode_FromString(ret);
+    g_free(ret);
+    return retval;
+}
+
 static PyMethodDef _glib_functions[] = {
+    { "threads_init",
+      (PyCFunction) pyglib_threads_init, METH_NOARGS,
+      "threads_init()\n"
+      "Initialize GLib for use from multiple threads. If you also use GTK+\n"
+      "itself (i.e. GUI, not just PyGObject), use gtk.gdk.threads_init()\n"
+      "instead." },
     { "idle_add",
       (PyCFunction)pyglib_idle_add, METH_VARARGS|METH_KEYWORDS,
       "idle_add(callable, user_data=None, priority=None) -> source id\n"
@@ -669,9 +703,11 @@ static PyMethodDef _glib_functions[] = {
     { "get_user_data_dir",
       (PyCFunction)pyglib_get_user_data_dir, METH_NOARGS },
     { "get_user_special_dir",
-      (PyCFunction)pyglib_get_user_special_dir, METH_VARARGS },
+      (PyCFunction)pyglib_get_user_special_dir, METH_VARARGS|METH_KEYWORDS },
     { "markup_escape_text",
       (PyCFunction)pyglib_markup_escape_text, METH_VARARGS|METH_KEYWORDS },
+    { "find_program_in_path",
+      (PyCFunction)pyglib_find_program_in_path, METH_VARARGS|METH_KEYWORDS },
     { NULL, NULL, 0 }
 };
 
